"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dgram_1 = require("dgram");
const dotup_ts_types_1 = require("dotup-ts-types");
const events_1 = require("events");
const os_1 = __importDefault(require("os"));
const SsdpConstants_1 = require("../SsdpConstants");
const SsdpMessage_1 = require("../Messages/SsdpMessage");
const SsdpSocketEvents_1 = require("./SsdpSocketEvents");
const tools_1 = require("../tools");
const logger = tools_1.getNamedLogger(tools_1.LOGGER_NAME);
class SsdpSocket extends events_1.EventEmitter {
    constructor(...interfaceNames) {
        super();
        this.interfaceNames = interfaceNames.length > 0 ? interfaceNames : undefined;
    }
    publish(message) {
        this.sockets.forEach(s => {
            message = dotup_ts_types_1.StringTools.format(message, { ip: s.ip, hostname: tools_1.getHostname() });
            s.send(message, SsdpConstants_1.SsdpConstants.port, SsdpConstants_1.SsdpConstants.ipv4Address, (e, b) => {
                if (e !== undefined && e !== null) {
                    logger.error(e);
                }
            });
        });
    }
    isConnected() {
        return this.connected;
    }
    initialize() {
        const interfaces = os_1.default.networkInterfaces();
        const allKeys = Object.keys(interfaces);
        let keys = allKeys;
        if (this.sockets === undefined) {
            this.sockets = [];
        }
        else {
            return;
        }
        if (this.interfaceNames !== undefined) {
            const notFound = this.interfaceNames.filter(name => interfaces[name] === undefined);
            notFound.forEach(name => logger.warn(`Interface '${name}' not found.`));
            keys = allKeys.filter(name => this.interfaceNames.includes(name));
        }
        keys.forEach(name => this.createSocket(name, interfaces[name]));
    }
    createSocket(name, netInfo) {
        netInfo.forEach(info => {
            if (info.internal || info.family === 'IPv6') {
                return;
            }
            const type = info.family === 'IPv4' ? 'udp4' : 'udp6';
            const addr = type === 'udp4' ? '0.0.0.0' : '::';
            const ssdpAddr = type === 'udp4' ? SsdpConstants_1.SsdpConstants.ipv4Address : SsdpConstants_1.SsdpConstants.ipv6AddressLinkLocal;
            const multiAddr = type === 'udp4' ? info.address : `::%${info.scopeid}`;
            const opt = {
                type: type,
                reuseAddr: true
            };
            const socket = dgram_1.createSocket(opt);
            socket.ip = info.address;
            socket.interfaceName = name;
            socket.on('message', (m, r) => {
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.message, m, r);
                this.onMessage(m, r);
            });
            socket.on('error', e => this.emit(SsdpSocketEvents_1.SsdpSocketEvents.error, e));
            socket.bind(SsdpConstants_1.SsdpConstants.port, info.address, () => {
                socket.addMembership(ssdpAddr, info.address);
                socket.setMulticastTTL(SsdpConstants_1.SsdpConstants.ttl);
                socket.setMulticastLoopback(true);
                socket.setBroadcast(true);
            });
            socket.on('listening', () => {
                this.connected = true;
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.listening, socket);
                logger.info(`Socket ${socket.ip} started listning`);
            });
            this.sockets.push(socket);
        });
    }
    onMessage(msg, rinfo) {
        logger.debug(msg.toString());
        const rows = msg.toString().split('\r\n');
        const m = new SsdpMessage_1.SsdpMessage();
        const startLine = rows.shift();
        rows.forEach(row => {
            if (row.trim().length > 0) {
                const split = row.indexOf(':');
                const key = row.substr(0, split).trim().toUpperCase();
                const value = row.substr(split + 1).trim();
                m[key] = value;
            }
        });
        switch (startLine) {
            case SsdpConstants_1.SsdpConstants.StartLine['M-SEARCH']:
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.msearch, m, rinfo);
                break;
            case SsdpConstants_1.SsdpConstants.StartLine.NOTIFY:
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.notify, m);
                break;
        }
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    dispose() {
        if (this.sockets !== undefined) {
            this.sockets.forEach(s => {
                s.close();
                s.unref();
            });
            this.sockets = undefined;
            this.connected = false;
        }
    }
}
exports.SsdpSocket = SsdpSocket;

//# sourceMappingURL=SsdpSocket.js.map
