"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dotup_ts_types_1 = require("dotup-ts-types");
const timers_1 = require("timers");
const MessageFactory_1 = require("./Messages/MessageFactory");
const SsdpConfigLoader_1 = require("./SsdpConfigLoader");
const SsdpSocketEvents_1 = require("./Sockets/SsdpSocketEvents");
const dgram_1 = require("dgram");
const SsdpConstants_1 = require("./SsdpConstants");
const tools_1 = require("./tools");
const logger = tools_1.getNamedLogger(tools_1.LOGGER_NAME);
class SsdpAdvertiser {
    constructor(socket, ssdpConfig) {
        this.socket = socket;
        this.ssdpConfig = ssdpConfig;
    }
    startPublishing() {
        if (this.timer !== undefined) {
            return;
        }
        this.socket.on(SsdpSocketEvents_1.SsdpSocketEvents.msearch, (m, r) => this.searchResponse(m, r));
        this.sendDiscoveryMessages();
        this.timer = setInterval(() => {
            this.sendDiscoveryMessages();
        }, (this.ssdpConfig.SsdpAdvertiser.cacheControl / 2) * 1000);
    }
    async stopPublishing() {
        if (this.timer !== undefined) {
            timers_1.clearTimeout(this.timer);
            this.timer = undefined;
        }
        this.socket.removeListener(SsdpSocketEvents_1.SsdpSocketEvents.msearch, this.searchResponse);
        await this.sendByebyeMessages();
    }
    sendDiscoveryMessages() {
        const factory = new MessageFactory_1.MessageFactory(this.ssdpConfig);
        const discoveryMessages = this.createMessages((ssdpDescriptor) => factory.createAliveMessage(ssdpDescriptor));
        discoveryMessages.forEach(async (m) => {
            await dotup_ts_types_1.sleep(50);
            this.publish(m);
        });
    }
    async sendByebyeMessages() {
        const factory = new MessageFactory_1.MessageFactory(this.ssdpConfig);
        const discoveryMessages = this.createMessages((ssdpDescriptor) => factory.createByebyeMessage(ssdpDescriptor));
        for (const m of discoveryMessages) {
            await dotup_ts_types_1.sleep(50);
            this.publish(m);
        }
    }
    createMessages(messageFactory) {
        const discoveryMessages = [];
        // Load config
        const rootDevice = SsdpConfigLoader_1.SsdpConfigLoader.createRootDeviceDescriptor(this.ssdpConfig);
        const rootItems = messageFactory(rootDevice);
        discoveryMessages.push(...rootItems);
        let items = rootDevice.services.map(s => messageFactory(s));
        if (items.length > 0) {
            discoveryMessages.push(...items.reduce((p, c) => p.concat(c)));
        }
        items = rootDevice.devices.map(d => {
            const devices = messageFactory(d);
            const deviceServices = d.services.map(ds => messageFactory(ds));
            return devices.concat(...deviceServices);
        });
        if (items.length > 0) {
            discoveryMessages.push(...items.reduce((p, c) => p.concat(c)));
        }
        return discoveryMessages;
    }
    publish(message) {
        this.socket.publish(message);
    }
    searchResponse(message, rinfo) {
        switch (message.ST) {
            case SsdpConstants_1.SsdpConstants.upnpRootDevice:
            case SsdpConstants_1.SsdpConstants.ssdpAll:
                const factory = new MessageFactory_1.MessageFactory(this.ssdpConfig);
                const discoveryMessages = this.createMessages((ssdpDescriptor) => factory.createAliveMessage(ssdpDescriptor));
                const socket = dgram_1.createSocket({
                    type: rinfo.family === 'IPv4' ? 'udp4' : 'udp6'
                });
                discoveryMessages.forEach(response => {
                    socket.send(response, 1900, rinfo.address, (error, bytes) => {
                        if (error !== undefined && error !== null) {
                            logger.error(error);
                        }
                    });
                });
                socket.close();
                break;
            default:
                break;
        }
    }
    async dispose() {
        if (this.socket === undefined) {
            return;
        }
        await this.stopPublishing();
        this.socket = undefined;
    }
}
exports.SsdpAdvertiser = SsdpAdvertiser;

//# sourceMappingURL=SsdpAdvertiser.js.map
