"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SsdpSocketEvents_1 = require("./Sockets/SsdpSocketEvents");
const SsdpConstants_1 = require("./SsdpConstants");
const events_1 = require("events");
const timers_1 = require("timers");
const SsdpControlPointEvents_1 = require("./SsdpControlPointEvents");
const SsdpMessage_1 = require("./Messages/SsdpMessage");
class SsdpControlPoint extends events_1.EventEmitter {
    constructor(socket) {
        super();
        this.socket = socket;
        this.items = [];
    }
    initialize() {
        this.socket.on(SsdpSocketEvents_1.SsdpSocketEvents.notify, msg => {
            switch (msg.NTS) {
                case SsdpConstants_1.SsdpConstants.ssdpAlive:
                    this.addOrUpdateItem(msg);
                    break;
                case SsdpConstants_1.SsdpConstants.ssdpByebye:
                    this.removeItem(msg);
                    break;
            }
        });
        this.timer = timers_1.setInterval(() => {
            this.cleanUp();
        }, 1000);
    }
    removeItem(msg) {
        const item = msg instanceof SsdpMessage_1.SsdpMessage ? this.parseUsn(msg) : msg;
        if (item !== undefined) {
            const index = this.items.indexOf(item);
            if (index >= 0) {
                this.items.splice(index, 1);
                this.emit(SsdpControlPointEvents_1.SsdpControlPointEvents.lost, item);
            }
        }
    }
    addOrUpdateItem(msg) {
        const item = this.parseUsn(msg);
        if (item === undefined) {
            return;
        }
        const index = this.items.findIndex(x => x.uuid === item.uuid &&
            x.domain === item.domain &&
            x.name === item.name &&
            x.type === item.type);
        if (index < 0) {
            this.items.push(item);
            this.emit(SsdpControlPointEvents_1.SsdpControlPointEvents.found, item, msg);
        }
        else {
            const entry = this.items[index];
            entry.timestamp = item.timestamp;
        }
    }
    // 'USN: uuid:9999::urn:dotup:device:motobox'
    // 'USN: uuid:9999::urn:dotup:service:mqtt'
    parseUsn(msg) {
        const usn = msg.USN.split('::');
        if (usn.length < 2) {
            return undefined;
        }
        let uuid = usn[0];
        uuid = uuid.substr(uuid.lastIndexOf(':') + 1).trim();
        const urn = usn[1].split(':');
        if (urn.length < 3) {
            return undefined;
        }
        const validity = msg['CACHE-CONTROL'] === undefined ? 0 : Number(msg['CACHE-CONTROL'].split('=')[1]);
        return {
            uuid: uuid,
            domain: urn[1],
            type: urn[2],
            name: urn[3],
            timestamp: new Date().toUTCString(),
            validitySEC: validity,
            USN: msg.USN
        };
    }
    // discover(): void {
    // }
    cleanUp() {
        const remove = [];
        this.items.forEach(item => {
            const currentDate = new Date(new Date().toUTCString());
            const itemDate = new Date(item.timestamp);
            const diff = currentDate.getUTCSeconds() - itemDate.getUTCSeconds();
            if (diff > Number(item.validitySEC)) {
                // Invalid
                remove.push(item);
            }
        });
        remove.forEach(item => {
            this.removeItem(item);
        });
    }
    search() {
        throw new Error('not implemented');
    }
    // tslint:disable-next-line: no-unnecessary-override
    on(event, listener) {
        return super.on(event, listener);
    }
    dispose() {
        if (this.timer !== undefined) {
            clearInterval(this.timer);
        }
    }
}
exports.SsdpControlPoint = SsdpControlPoint;

//# sourceMappingURL=SsdpControlPoint.js.map
